pub enum BlissError {
DecodingError(String),
AnalysisError(String),
ProviderError(String),
}Expand description
Umbrella type for bliss error types
Variants
DecodingError(String)
An error happened while decoding an (audio) file.
AnalysisError(String)
An error happened during the analysis of the song’s samples by bliss.
ProviderError(String)
An error happened with the music library provider. Useful to report errors when you implement bliss for an audio player.