pub struct BaseConfig {
pub config_path: PathBuf,
pub database_path: PathBuf,
pub analysis_options: AnalysisOptions,
pub m: Array2<f32>,
}Expand description
The minimum configuration an application needs to work with a Library.
Fields
config_path: PathBufThe path to where the configuration file should be stored,
e.g. /home/foo/.local/share/bliss-rs/config.json
database_path: PathBufThe path to where the database file should be stored,
e.g. /home/foo/.local/share/bliss-rs/bliss.db
analysis_options: AnalysisOptionsThe analysis options set in the database (number of CPU cores for the analysis, desired feature version…)
m: Array2<f32>The mahalanobis matrix used for mahalanobis distance.
Used to customize the distance metric beyond simple euclidean distance.
Uses ndarray’s serde feature for serialization / deserialization.
Field would look like this:
“m”: {“v”: 1, “dim”: [20, 20], “data”: [1.0, 0.0, …, 1.0]}
Implementations
Sourceimpl BaseConfig
impl BaseConfig
Sourcepub fn new(
config_path: Option<PathBuf>,
database_path: Option<PathBuf>,
analysis_options: Option<AnalysisOptions>,
) -> Result<Self>
pub fn new( config_path: Option<PathBuf>, database_path: Option<PathBuf>, analysis_options: Option<AnalysisOptions>, ) -> Result<Self>
Create a new, basic config. Upon calls of Config.write(), it will be
written to config_path.
Any path omitted will instead default to a “clever” path using
data directory inference. The “clever” thinking is as follows:
- If the user specified only one of the paths, it will put the other file in the same folder as the given path.
- If the user specified both paths, it will go with what the user chose.
- If the user didn’t select any path, it will try to put everything in the user’s configuration directory, i.e. XDG_CONFIG_HOME.
The number of cores is the number of cores that should be used for any analysis. If not provided, it will default to the computer’s number of cores.