pub struct ProcessingError {
pub song_path: PathBuf,
pub error: String,
pub features_version: FeaturesVersion,
}Expand description
Hold an error that happened while processing songs during analysis.
Fields
song_path: PathBufThe path of the song whose analysis was attempted.
error: StringThe actual error string.
features_version: FeaturesVersionFeatures version the analysis was attempted with.