pub enum SymphoniaDecoderError {
ResampleError(String),
ResamplerConstructionError(String),
IoError(String),
DecodeError(String),
UnsupportedCodec,
NoSupportedAudioTracks,
NoStreams,
IndeterminantDuration,
}Expand description
Error raised when trying to decode a song with the SymphoniaDecoder.
Variants
ResampleError(String)
Error raised when trying to resample audio. (from rubato)
ResamplerConstructionError(String)
Error raised when trying to create a resampler. (from rubato)
IoError(String)
General IO error.
DecodeError(String)
Error raised when trying to decode audio. (from symphonia)
UnsupportedCodec
Error raised when trying to decode a file with an unsupported codec.
NoSupportedAudioTracks
Error raised when trying to decode a file with no supported audio tracks.
NoStreams
Error raised when trying to decode a file with no streams.
IndeterminantDuration
Error raised when the audio source’s duration is either unknown or infinite.