pub enum FeaturesVersion {
Version2 = 2,
Version1 = 1,
}Expand description
The versions of the features used for analysis. Used for backwards-compatibility reasons in case people want to keep using older features version.
Songs analyzed with different FeaturesVersion are not compatible with one another, as they might have a different set of features, etc.
Variants
Version2 = 2
The latest iteration, increasing chroma features accuracy and making feature normalization more coherent.
Version1 = 1
The first iteration of the features. The 4 last chroma features (song mode detection) might underperform / be underused while computing distances.
Implementations
Sourceimpl FeaturesVersion
impl FeaturesVersion
Sourcepub const LATEST: FeaturesVersion = FeaturesVersion::Version2
pub const LATEST: FeaturesVersion = FeaturesVersion::Version2
Always points to the latest features’ version. In case of doubt, use this one.
Sourcepub const fn feature_count(self) -> usize
pub const fn feature_count(self) -> usize
Number of features for this version (usable in const contexts).