pub struct LibrarySong<T: Serialize + DeserializeOwned + Clone> {
pub bliss_song: Song,
pub extra_info: T,
}Expand description
Struct holding both a Bliss song, as well as any extra info that a user would want to store in the database related to that song.
The only constraint is that extra_info must be serializable, so,
something like
#[derive(Serialize)]
struct ExtraInfo {
ignore: bool,
unique_id: i64,
}
let extra_info = ExtraInfo { ignore: true, unique_id = 123 };
let song = LibrarySong { bliss_song: song, extra_info };is totally possible.
Fields
bliss_song: SongActual bliss song, containing the song’s metadata, as well as the bliss analysis.
extra_info: TUser-controlled information regarding that specific song.