pub struct Analysis {
pub features_version: FeaturesVersion,
/* private fields */
}Expand description
Object holding the results of the song’s analysis.
Only use it if you want to have an in-depth look of what is happening behind the scene, or make a distance metric yourself.
Under the hood, it is just an array of f32 holding different numeric features.
For more info on the different features, build the
documentation with private items included using
cargo doc --document-private-items, and / or read up
this document, that contains a description
on most of the features, except the chroma ones, which are documented
directly in this code.
Fields
features_version: FeaturesVersionIt is bumped every time a change is introduced in the features that makes them incompatible with previous versions.
Implementations
Sourceimpl Analysis
impl Analysis
Sourcepub fn new(
analysis: Vec<f32>,
features_version: FeaturesVersion,
) -> BlissResult<Analysis>
pub fn new( analysis: Vec<f32>, features_version: FeaturesVersion, ) -> BlissResult<Analysis>
Create a new Analysis object.
Usually not needed, unless you have already computed and stored features somewhere, and need to recreate a Song with an already existing Analysis yourself.